#ifndef TBInOut_H
#define TBInOut_H


#include "CDSys.h"



/*  The module InOut provides procedures for reading and writing
    basic types to the standard input and standard output units.

    Standard input defaults to the user's terminal keyboard.
    Standard output defaults to the user's terminal screen.
    Both can be redirected to other files (see procedures
    OpenInput and OpenOutput).

    Status from the various operations provided by module InOut
    is communicated through the varaible "Done" which should be imported
    by any module using InOut.

    "Done" is set after every read operation:

       true indicates that the last operation was successfully completed.

       false indicates that the previous operation failed (either
             because the end-of-stream was reached or because of
             an error.

                                                                      */

#define EOL             '\015'


extern Boolean Done_;   /* indicates result of last read operation */
extern char termCH;   /* terminating character in ReadInt, ReadCard */
extern short in, out;   /* for exceptional cases only */



/*Request a file name and open that file as the standard input
 file "in". If opened successfully, subsequent input is read
 from this file.

  Done := "file was successfully opened".

  If the name entered by the user ends with ".",  the default
  extension given by 'defext is appended to the filename. */


pQUALIFIER void OpenInput(char defext[]);

/*Request a file name and open that output file as the standard
  output file "out".If opened successfully, subsequent output is
   written  to this file.

  Done := "file was successfully opened.

  If the name entered by the user ends with ".",  the default
  extension given by 'defext is appended to the filename.

  NOTE: Output to the standard output is not echoed to the terminal
        when output has been directed to a file.                                                                                            */

pQUALIFIER void OpenOutput(char defext[]);

/*closes the input file; returns standard input to the terminal*/

pQUALIFIER void CloseInput(void);
/*closes the output file; returns standard output to the terminal*/

pQUALIFIER void CloseOutput(void);

/* Procedure Read waits for and returns in "ch" the next
   character from the standard input.

   Done := NOT in.eof
                                                               */

pQUALIFIER void Read(char *ch);
/* Read in a string, i.e. sequence of characters not containing
   blanks or control characters from the standard input.
   Leading blanks are ignored.

   Input is terminated by any character <= " " (space);
   this character is assigned to termCH.

   BACKSPACE is used for backspacing when input from terminal   */


pQUALIFIER void ReadString(char *s);

/*Read  in a string and convert it to an integer and assign its
  value to "x".

  Integers may by preceded by a "+' or a "-" sign.

  Leading blanks are ignored.

  Done := "integer was read"                                    */


pQUALIFIER void ReadInt(short *x);

/*Read in a string and convert it to a cardinal and assign its
  value to "x".

  Leading blanks are ignored.

  Done := "cardinal was read"                                    */


pQUALIFIER void ReadCard(unsigned short *x);

/* Read  the next word from the standard input and assign it
   to "w".

   Done := NOT in.eof                                             */

pQUALIFIER void ReadWrd(long *w);

/* Procedure Write outputs the character "ch" to the standard output */

pQUALIFIER void Write(char ch);

/* terminate the current line on the standard output
   this is the same as Write(EOL)                               */

pQUALIFIER void WriteLn(void);

/* Procedure WriteString writes out the given string to the
   the standard output. The string to be written should be
   terminated with the character 0C (null)                      */

pQUALIFIER void WriteString(char s[]);

/* Procedure WriteInt writes an integer "x" with (at least)
   n characters to the standard output.

   If "n" is greater than the number of digits needed to express
   the number, leading  blanks are added preceding the number
   to fill the requested space                                   */

pQUALIFIER void WriteInt(short x, unsigned short n);

/* Procedure WriteCard writes a cardinal "x" with (at least)
   n characters to the standard output.

   If "n" is greater than the number of digits needed to express
   the number, leading  blanks are added preceding the number
   to fill the requested space                                   */

pQUALIFIER void WriteCard(unsigned short x, unsigned short n);

/* Procedure WriteOct writes a cardinal "x" with (at least)
   n characters to the standard output in octal.

   If "n" is greater than the number of digits needed to express
   the number, leading  blanks are added preceding the number
   to fill the requested space                                   */


pQUALIFIER void WriteOct(unsigned short x, unsigned short n);

/* Procedure WriteHex writes a cardinal "x" with (at least)
   n characters to the standard output in hexadecimal.

   If "n" is greater than the number of digits needed to express
   the number, leading  blanks are added preceding the number
   to fill the requested space                                   */


pQUALIFIER void WriteHex(unsigned short x, unsigned short n);

/* Procedure WriteWrd outputs the word "w" to the standard output */

pQUALIFIER void WriteWrd(long w);

/* Clears the current graph port. */

pQUALIFIER void ClearScreen(void);

pQUALIFIER void TBInOut_Init(void);


#endif /*TBInOut_H*/

/* End. */
